' -*- Mode: Text; tab-width: 2; indent-tabs-mode: nil;  -*- 
{{TCU_VoySpeedup.spin}}
'
{{
CHANGES:
29Sep16: Starting with Repa Voyantic Speedup, implement multiport serial features,
and set up for concurrent lane operations, including U+HF in one lane with two mark deltas.
This is a rewrite of the Target subsystem relative to the historicat GlyeLogix TCU:
  * multiple triggers
  * driven by assembly level state machine
  * concurrent operations
.
02Mar2016: Repa speedup, match fast Voyantic to slow EzPLC
}}

DAT
  'Version and commands
  _hello byte "FourSerialX2Test_03Oct2016"     'ADJUST_VERSION_HERE
  _crlf byte 13, 10, 0
  _hex byte "0123456789ABCDEF", 0
  _HostOpcodes byte "QOVRrMmSsTtPpLKGZ:;>", 0 'always put new opcodes at end in case position is important to some piece of code

CON
  'System Parameters
  _CLKMODE = XTAL1 + PLL16X
  _XINFREQ = 5_000_000                     'Algorithms depend on code being >> faster than data
  'Serial
  pLLIOHostTx                  = 0
  pLLIOHostRx                  = 1
  pLLIOHostDtr                 = 2
  pLLIODb9Tx                   = 18 'historical Target port on V3
  pLLIODb9Rx                   = 17
  pLLIOConn12aTx               = 15 'historical Barcode port
  pLLIOConn12aRx               = 14
  pLLIOConn12bTx               = 21 'historical HFTarget port
  pLLIOConn12bRx               = 19
  pLLIONewBarcodeTx            = 20  ' wire to TP36
  pLLIONewBarcodeRx            = 16  'wire to TP33
  pLLIOVprnRx                  = 22  'PIO22 U3.R5out (rx) TargetRI232 from TP34
  pLLIOVprnTx                  = 23  'PIO23 unused TP20, jump TP20-TP49, wire DB9 to TP42
  pLLIODebugUsbTx              = 30
  pLLIODebugUsbRx              = 31
  kLLSerBufSize                = 128
  ixLLSB_Opcode                = 0
  ixLLSB_Hex1                  = 1
  ixLLSB_Hex2                  = 2
  ixLLSB_XorByte               = 3
  ixLLSB_CrLf                  = 4
  kTargetBaud                  = 38400
  'AuxSerial cog ports
  kAuxSpDebugUsb               = 0
  kAuxSpBarcode                = 1
  kAuxSpPrint                  = 2
  'FourReader cog ports
  k4RdrDb9                      = 0
  k4RdrConn12a                  = 1
  k4RdrConn12b                  = 2
  k4RdrUnused                   = 3  

VAR
  long lMsecCounter, lClockCtrAtLastMsec
  Byte LLSerBuf[kLLSerBufSize] 'Host
  Byte bLLSerBufDatalen        'Host

OBJ
  ' Note - do not set debug signals like OUTA[11] in code modules 
  '   that run in more than one cog.  The signals will interfere.
  Host : "BB_FullDuplexSerial"
  FourRdrs : "FourSerial"
  AuxSerial : "FourSerial"

PUB _Entry
  MAIN


'NOTE, THIS IS NOT AN INTERRUPT
PUB TimerMain | now, dt
  'at 5MHz and PLL8x, the main loop takes 2..3 mS.  SX48 assumptions about speed >> reality are not true.
  'Sep11 this is very approximate due to loop delay.  On idle system, pulses at 1.3 mS; on active system, longer
  'CNT wraps every 5 seconds or so, so direct computations on CNT are not feasible.
  now := CNT                  'get this once since CNT will change during this function
  '|| is Absolute Value
  dt := (||(now - lClockCtrAtLastMsec)) / (CLKFREQ/1_000)
  if dt > 0
    'millisecond boundary
    lClockCtrAtLastMsec += (dt * (CLKFREQ/1_000))      'Jul13 correct rounding error, was := now
    lMsecCounter += dt
  'not sure about signed/unsigned behavior, so programatically wrap
  if lMsecCounter < 0
    lMsecCounter ~

PUB Main | hscog, llcog, ascog, r4cog
  'Jan16LM: AgileBaud
  hscog := Host.start(pLLIOHostRx, pLLIOHostTx, 0, 38400)
  AuxSerial.AddPort(kAuxSpDebugUsb, pLLIODebugUsbRx, pLLIODebugUsbTx, -1, -1, -1, 0, kTargetBaud)
  AuxSerial.AddPort(kAuxSpBarcode, pLLIONewBarcodeRx, pLLIONewBarcodeTx, -1, -1, -1, 0, kTargetBaud)
  AuxSerial.AddPort(kAuxSpPrint, pLLIOVprnRx, pLLIOVprnTx, -1, -1, -1, 0, kTargetBaud)
  ascog := AuxSerial.AddPort(3, 24, 25, -1, -1, -1, 0, kTargetBaud) 'Do we need all 4 ports running?
  'Comment this line and teh software runs ok 5x5
  'Uncomment this line and the software fails every time:
  ' * no serial output
  ' * no OUT10 toggle
  AuxSerial.Start
  FourRdrs.AddPort(k4RdrDb9, pLLIODb9Rx, pLLIODb9Tx, -1, -1, -1, 0, kTargetBaud)
  FourRdrs.AddPort(k4RdrConn12a, pLLIOConn12aRx, pLLIOConn12aTx, -1, -1, -1, 0, kTargetBaud)
  FourRdrs.AddPort(k4RdrConn12b, pLLIOConn12bRx, pLLIOConn12bTx, -1, -1, -1, 0, kTargetBaud)
  FourRdrs.AddPort(3, 24, 25, -1, -1, -1, 0, kTargetBaud) 'Do we need all 4 ports running?
  r4cog := FourRdrs.Start
  Host.Str(string("COGS: "))
  Host.Dec(hscog)
  Host.Tx(" ")
  Host.Dec(ascog)
  Host.Tx(" ")
  Host.Dec(r4cog)
  Host.Str(@_crlf) '2 3 0 4
  'I/O: ~ is unary post-clear, ~~ is unary post-set
  DIRA~                           'clear all to input, as after reset
  DIRA[10] ~~
  'Send Version Banner
  Host.str(@_hello)
  'Main Loop
  repeat 'always
    OUTA[10] ~~
    TimerMain
    OUTA[10] ~
    Host.str(@_hello)
    'end repeat always
  'these lines should never execute
  Host.str(string("Main Loop Exited, Rebooting!",13,10))
  waitcnt (CLKFREQ + cnt)                                'make sure port stays open long enough to shift out
  'Reset the chip
  REBOOT 'clkset($80, 0)
